!---------------------------------------------------------------------
!- initialize for fft program
!---------------------------------------------------------------------
      subroutine fftini()
      include 'head.h'

      real*8 t(8*nmax)
      real*8 w(nmax*3/2)
      common /fftcoef1/ t, w
      integer ip(nmaxsqrt)
      common /fftcoef2/ ip

      ip(1) = 0

      return
      end
!---------------------------------------------------------------------
!- real to complex
!---------------------------------------------------------------------
      subroutine rc(work)
      include 'head.h'

      real*8 work(nx, ny, nz)

      real*8 t(8*nmax)
      real*8 w(nmax*3/2)
      common /fftcoef1/ t, w
      integer ip(nmaxsqrt)
      common /fftcoef2/ ip

      real*8 tmp1
      integer ix, iy, iz

      call rdft3d(nx, ny, nx, ny, nz, 1, work, t, ip, w)

      tmp1 = 2.0d0/dble(nx*ny*nz)
      do iz = 1, nz
         do iy = 1, ny
         do ix = 1, nx
         work(ix, iy, iz) = work(ix, iy, iz)*tmp1
         end do
         end do
      end do

      return
      end
!---------------------------------------------------------------------
!- real to complex for 3d vector
!---------------------------------------------------------------------
      subroutine rc_3d(u)
      include 'head.h'

      real*8 u(nx, ny, nz, 3)

      integer i

      do i = 1, 3
      call rc(u(1,1,1,i))
      end do

      return
      end
!---------------------------------------------------------------------
!- complex to real
!---------------------------------------------------------------------
      subroutine cr(work)
      include 'head.h'

      real*8 work(nx, ny, nz)

      real*8 t(8*nmax)
      real*8 w(nmax*3/2)
      common /fftcoef1/ t, w
      integer ip(nmaxsqrt)
      common /fftcoef2/ ip

      call rdft3d(nx, ny, nx, ny, nz, -1, work, t, ip, w)

      return
      end
!---------------------------------------------------------------------
!- complex ro real for 3d vector
!---------------------------------------------------------------------
      subroutine cr_3d(u)
      include 'head.h'

      real*8 u(nx, ny, nz, 3)

      integer i

      do i = 1, 3
         call cr(u(1,1,1,i))
      end do

      return
      end
!---------------------------------------------------------------------
!- truncate by 1/3 rule
!---------------------------------------------------------------------
      subroutine truncate1o3(a, nx, ny, nz)

      integer nx, ny, nz
      complex*16 a(nx/2, ny, nz)

      integer ix, iy, iz
      complex*16 zero
      real*8 tmp

      zero = dcmplx(0.0d0, 0.0d0)

!---- some specials because of the fft program ordering
      tmp = dble(a(1, 1, 1))
      a(1, 1, 1) = dcmplx(tmp, 0.0d0)
      do iz = nz/2+2, nz
         a(1, 1, iz) = zero
      end do

!---- x direction ----
      do iz = 1, nz
      do iy = 1, ny
      do ix = nx/3-1, nx/2
         a(ix, iy, iz) = zero
      end do
      end do
      end do

!---- y direction ----
      do iz = 1, nz
      do iy = ny/3-1, ny/2
      do ix = 1, nx/2
         a(ix, iy, iz) = zero
      end do
      end do
      end do

      do iz = 1, nz
      do iy = ny/2+1, ny*2/3+1
      do ix = 1, nx/2
         a(ix, iy, iz) = zero
      end do
      end do
      end do

!---- z direction ----
      do iz = nz/3-1, nz/2
      do iy = 1, ny
      do ix = 1, nx/2
         a(ix, iy, iz) = zero
      end do
      end do
      end do

      do iz = nz/2+1, nz*2/3+1
      do iy = 1, ny
      do ix = 1, nx/2
         a(ix, iy, iz) = zero
      end do
      end do
      end do

      return
      end
!---------------------------------------------------------------------
!- truncate scalar
!---------------------------------------------------------------------
      subroutine truncate(work)
      include 'head.h'

      complex*16 work(nx/2, ny, nz)

      call truncate1o3(work, nx, ny, nz)

      return
      end
!---------------------------------------------------------------------
!- truncate a 3d vector
!---------------------------------------------------------------------
      subroutine truncate_3d(u)
      include 'head.h'

      complex*16 u(nx/2, ny, nz, 3)

      integer i

      do i = 1, 3
         call truncate1o3(u(1,1,1,i), nx, ny, nz)
      end do

      return
      end
!---------------------------------------------------------------------
!- calculate pdx
!---------------------------------------------------------------------
      subroutine pdx(a, nx, ny, nz)

      integer nx, ny, nz
      complex*16 a(nx/2, ny, nz)

      complex*16 ci
      integer ix, iy, iz

      real*8 kx

      ci = dcmplx(0.0d0, -1.0d0)
      do iz = 1, nz
      do iy = 1, ny
      do ix = 1, nx/2
         kx = dble(ix-1)
         a(ix, iy, iz) = kx*ci*a(ix, iy, iz)
      end do
      end do
      end do

      return
      end
!---------------------------------------------------------------------
!- calculate pdy
!---------------------------------------------------------------------
      subroutine pdy(a, nx, ny, nz)

      integer nx, ny, nz
      complex*16 a(nx/2, ny, nz)

      complex*16 ci
      integer ix, iy, iz

      real*8 ky

      ci = dcmplx(0.0d0, -1.0d0)

      do iz = 1, nz
      do iy = 1, ny
      ky = dble(iy-1)
      if (iy .gt. ny/2) then
         ky = ky-dble(ny)
      end if
      do ix = 1, nx/2
         a(ix, iy, iz) = ky*ci*a(ix, iy, iz)
      end do
      end do
      end do

      return
      end
!---------------------------------------------------------------------
!- calculate pdz
!---------------------------------------------------------------------
      subroutine pdz(a, nx, ny, nz)

      integer nx, ny, nz
      complex*16 a(nx/2, ny, nz)

      complex*16 ci
      integer ix, iy, iz

      real*8 kz

      ci = dcmplx(0.0d0, -1.0d0)

      do iz = 1, nz
      kz = dble(iz-1)
      if (iz .gt. nz/2) then
         kz = kz-dble(nz)
      end if
      do iy = 1, ny
      do ix = 1, nx/2
         a(ix, iy, iz) = kz*ci*a(ix, iy, iz)
      end do
      end do
      end do

      return
      end
!---------------------------------------------------------------------
!- copy origin(nx, ny, nz) to goal(nx, ny, nz)
!---------------------------------------------------------------------
      subroutine arraycopy(goal, origin)
      include 'head.h'

      real*8 goal(ntot), origin(ntot)

      integer i

      do i = 1, ntot
         goal(i) = origin(i)
      end do

      return
      end
!---------------------------------------------------------------------
!- copy origin(nx, ny, nz , 3) to goal(nx, ny, nz, 3)
!---------------------------------------------------------------------
      subroutine arraycopy_3d(goal, origin)
      include 'head.h'

      real*8 goal(ntot*3), origin(ntot*3)

      integer i

      do i = 1, ntot*3
         goal(i) = origin(i)
      end do

      return
      end
!---------------------------------------------------------------------
!- define wavevector k
!---------------------------------------------------------------------
      subroutine kvector(k)
      include 'head.h'

      real*8 k(nx/2, ny, nz, 3)

      integer j, ix, iy, iz

      real*8 rk

      real*8 spec(smax, 3)
      common /spectrumfield/ spec

      integer kpoint(nx/2, ny, nz)
      common /idwavenumber/ kpoint

      real*8 ispec(smax, 2)
      common /ispectrumfield/ ispec

      integer ikpoint(nx/2, ny, nz)
      common /iidwavenumber/ ikpoint

      do iz = 1, nz
      do iy = 1, ny
      do ix = 1, nx/2

      k(ix, iy, iz, 1) = dble(ix)-1.0d0
      k(ix, iy, iz, 2) = dble(iy)-1.0d0
      k(ix, iy, iz, 3) = dble(iz)-1.0d0
      if (iy .gt. ny/2) then
         k(ix, iy, iz, 2) = k(ix, iy, iz, 2)-dble(ny)
      end if
      if (iz .gt. nz/2) then
         k(ix, iy, iz, 3) = k(ix, iy, iz, 3)-dble(nz)
      end if

      end do
      end do
      end do

!      if (iforce > 0 .and. iforce < 4 .or. iforce .eq. 12) then
      if (iforce > 0 .and. iforce < 4) then
!      if (iforce < 4) then

         spec(1, 1) = 0.0
         spec(2, 1) = 1.0
         spec(3, 1) = 1.42
         spec(4, 1) = 1.74
         spec(5, 1) = 2.0
         spec(6, 1) = 2.24
         spec(7, 1) = 2.45
         spec(8, 1) = 2.83
         spec(9, 1) = 3.0
         spec(10, 1) = 3.17
         spec(11, 1) = 3.32
         spec(12, 1) = 3.5
         spec(13, 1) = 4.0
         spec(14, 1) = 4.5
         spec(15, 1) = 5.0
         spec(16, 1) = 5.5
         spec(17, 1) = 6.0
         spec(18, 1) = 6.5
         spec(19, 1) = 7.0
         spec(20, 1) = 7.5
         spec(21, 1) = 8.0
         spec(22, 1) = 8.5
         spec(23, 1) = 9.0
         spec(24, 1) = 9.5
         spec(25, 1) = 10.0
         spec(26, 1) = 10.5
         spec(27, 1) = 11.0
         spec(28, 1) = 11.5

         do ix = 29, smax
            spec(ix, 1) = (dble(ix)-29.0d0)*1.0d0+12.0d0
         end do

      else

!      spec(1, 1) = 0.0
!      spec(2, 1) = 1.0
!      spec(3, 1) = 1.42
!      do ix = 4, smax
!      spec(ix, 1) = (dble(ix)-4.0d0)*1.0d0+2.0d0
!      end do

         do ix = 1, smax
            spec(ix, 1) = dble(ix)-1.0d0
         end do

      end if

      do ix = 1, smax
         ispec(ix, 1) = dble(ix)-1.0d0
      end do

      do iz = 1, nz
      do iy = 1, ny
      do ix = 1, nx/2
         rk = k(ix,iy,iz,1)**2+k(ix,iy,iz,2)**2+k(ix,iy,iz,3)**2
         rk = dsqrt(rk)
         do j = 1, smax-1
            if (rk .ge. spec(j, 1) .and. rk .lt. spec(j+1, 1)) then
               kpoint(ix, iy, iz) = j
            end if
            if (rk .ge. ispec(j, 1) .and. rk .lt. ispec(j+1, 1)) then
               ikpoint(ix, iy, iz) = j
            end if
         end do
      end do
      end do
      end do

      return
      end
!---------------------------------------------------------------------
!- calculate the curl of a vector in fourier space:a = curl(b)
!---------------------------------------------------------------------
      subroutine curl(a, b)
      include 'head.h'

      complex*16 a(nx/2, ny, nz, 3), b(nx/2, ny, nz, 3)

!---- arrays of wavenumber ----
      real*8 k(nx/2, ny, nz, 3)
      common /wavenumber/ k

      complex*16 tmp1, tmp2, tmp3, ci

      integer ix, iy, iz

      ci = dcmplx(0.0d0, -1.0d0)

      do iz = 1, nz
      do iy = 1, ny
      do ix = 1, nx/2

         tmp1 = b(ix, iy, iz, 1)
         tmp2 = b(ix, iy, iz, 2)
         tmp3 = b(ix, iy, iz, 3)

         a(ix,iy,iz,1) = ci*(k(ix,iy,iz,2)*tmp3-k(ix,iy,iz,3)*tmp2)
         a(ix,iy,iz,2) = ci*(k(ix,iy,iz,3)*tmp1-k(ix,iy,iz,1)*tmp3)
         a(ix,iy,iz,3) = ci*(k(ix,iy,iz,1)*tmp2-k(ix,iy,iz,2)*tmp1)

      end do
      end do
      end do

      return
      end
!---------------------------------------------------------------------
!- reform the vector to be incompressible
!---------------------------------------------------------------------
      subroutine reform(a)
      include 'head.h'

      complex*16 a(nx/2, ny, nz, 3)

!---- arrays of wavenumber ----
      real*8 k(nx/2, ny, nz, 3)
      common /wavenumber/ k

      complex*16 temp
      real*8 rk2

      integer ix, iy, iz, i

      do iz = 1, nz
      do iy = 1, ny
      do ix = 1, nx/2

         temp = k(ix,iy,iz,1)*a(ix,iy,iz,1)
     &+k(ix,iy,iz,2)*a(ix,iy,iz,2)+k(ix,iy,iz,3)*a(ix,iy,iz,3)
         rk2 = k(ix,iy,iz,1)*k(ix,iy,iz,1)
     &+k(ix,iy,iz,2)*k(ix,iy,iz,2)+k(ix,iy,iz,3)*k(ix,iy,iz,3)

         a(ix,iy,iz,1) = a(ix,iy,iz,1)-temp*k(ix,iy,iz,1)/rk2
         a(ix,iy,iz,2) = a(ix,iy,iz,2)-temp*k(ix,iy,iz,2)/rk2
         a(ix,iy,iz,3) = a(ix,iy,iz,3)-temp*k(ix,iy,iz,3)/rk2

      end do
      end do
      end do

      ix = 1
      iy = 1
      iz = 1

      do i = 1, 3
         a(ix, iy, iz, i) = dcmplx(0.0d0, 0.0d0)
      end do

      return
      end
!---------------------------------------------------------------------
!- calculate the gradient of a scalar in fourier space:a = gra(b)
!---------------------------------------------------------------------
      subroutine gradient(a, b)
      include 'head.h'

      complex*16 a(nx/2, ny, nz, 3), b(nx/2, ny, nz)

!---- arrays of wavenumber ----
      real*8 k(nx/2, ny, nz, 3)
      common /wavenumber/ k

      complex*16 temp, ci

      integer ix, iy, iz

      ci = dcmplx(0.0d0, -1.0d0)

      do iz = 1, nz
      do iy = 1, ny
      do ix = 1, nx/2

         temp = ci*b(ix, iy, iz)
         a(ix, iy, iz, 1) = k(ix, iy, iz, 1)*temp
         a(ix, iy, iz, 2) = k(ix, iy, iz, 2)*temp
         a(ix, iy, iz, 3) = k(ix, iy, iz, 3)*temp

      end do
      end do
      end do

      return
      end
!-------------------------------------------------------------------------------
