!---------------------------------------------------------------------
!- output (x, y, z, u, v, w) to the file named filename
!- this output subroutine is for 'tecplot' ascii input formation
!---------------------------------------------------------------------
      subroutine velocity_output(u, filename)
      include 'head.h'

      real*8 u(nx, ny, nz, 3)
      character*99 filename

      real*8 tempu(nx, ny, nz, 3)
      common /vectortemp1/ tempu

      real*8 x, dx, y, dy, z, dz

      integer i, ix, iy, iz

      call arraycopy_3d(tempu, u)
      call cr_3d(tempu)

      dx = 2.0d0*pi/dble(nx)
      dy = 2.0d0*pi/dble(ny)
      dz = 2.0d0*pi/dble(nz)

      open(99, file = filename)

      write(99, *) 'title = "velocity profile @ time = ', time, ' "'
      write(99, *) 'variables = "x", "y", "z", "u", "v", "w"'
      write(99, 1) nx, ny, 1
1     format(1x,'zone i = ',i3,', j = ',i3,', k = ',i3,', f = point')

      z = -pi
      do iz = nz/2, nz/2
         y = -pi
         do iy = 1, ny
            x = -pi
            do ix = 1, nx
               write(99, 2) x, y, z, (tempu(ix, iy, iz, i), i = 1, 3)
2              format(6e17.8)
               x = x+dx
            end do
            y = y+dy
         end do
         z = z+dz
      end do

      close(99)

      return
      end
!---------------------------------------------------------------------
!- read all parameters from parameter file
!---------------------------------------------------------------------
      subroutine readparams()
      include 'head.h'

      character*99 c2, typef, typei, typed
      character*99 line, word, name
      integer i, k
      real*8 val

!- the head of parameters
      name = '+f'
      call tolower(name)
      typef = name

      name = '+i'
      call tolower(name)
      typei = name

      name = '+d'
      call tolower(name)
      typed = name

      write(*, *) 'opening parameter file...'
      name = 'param.param'
      open(99, status = 'old', form = 'formatted', file = name)

!----
!- read the first line of the form:
!- run #  mode #
!- where mode = 0 is normal mode.
!- mode = 1 is to just do a measurement, without opening any normal
!- output files.
!- (unused now)
!----
      read(99, '(a)') line

      call popword(line, word)
      call popword(line, word)
      read(word, '(i5)') nrun
      call popword(line, word)
      call popword(line, word)
      read(word, '(i5)') mode

!----
!- now read till eof
!----
      do k = 1, nparam

            read(99, '(a)', end = 10) line
            call popword(line, c2)

            if (c2 .eq. typei) then
                  call popword(line, name)
                  call popword(line, word)
                  read(word, *) i
                  call addiparam(name, i)
            end if

            if (c2 .eq. typed) then
                  call popword(line, name)
                  call popword(line, word)
                  read(word, *) val
                  call adddparam(name, val)
            end if

      end do

10    close(99)

      call setparameters() !setup the all parameters

      if (dabs(rnu) < 1.0d-20) then
         rnu = 1.0d-20
      end if

      if (irotate .ne. 1) then
         romega = 0.0d0
      end if

      return
      end
!---------------------------------------------------------------------
!- convert string to lower case
!---------------------------------------------------------------------
      subroutine tolower(string)
      character*99 string
      integer ilen, i, shift, ia, iz

      shift = ichar('s')-ichar('S')
      ia = ichar('A')
      iz = ichar('Z')

      ilen = len_trim(string)
      do i = 1, ilen
         ic = ichar(string(i:i))
         if (ic .ge. ia .and. ic .le. iz) then
            ic = ic+shift
            string(i:i) = char(ic)
         end if
      end do

      return
      end
!---------------------------------------------------------------------
!- give the first word of string,
!- for example, originally str = 'sun is shining.'
!- call popword(str, word), word = 'sun'
!- call popword(str, word), word = 'is'
!- call popword(str, word), word = 'shining.'
!---------------------------------------------------------------------
      subroutine popword(line, word)
      character*99 line, word
      integer ilen, k, i

      call stripspace(line)

      ilen = len(line)
      k = 0
      do i = 1, ilen
         if(line(i:i) .eq. ' ') then
            goto 151
         else
            k = k+1
         end if
      end do
151   do i = 1, k
      word(i:i) = line(i:i)
      end do
      do i = k+1, 99
      word(i:i) = ' '
      end do

      do i = 1, ilen-k-1
      line(i:i) = line(k+1+i:k+1+i)
      end do

      return
      end
!---------------------------------------------------------------------
!- for example, converting string '   word  ' = = >'word  '
!---------------------------------------------------------------------
      subroutine stripspace(line)
      character*99 line
      integer ilen, k, i

      ilen= len(line)
      k = 0
      do i = 1, ilen
         if(line(i:i) .eq. ' ') then
            k = k+1
         else
            goto 10
         end if
      end do
10    continue
      do i = 1, ilen-k
         line(i:i) = line(k+i:k+i)
      end do

      return
      end
!---------------------------------------------------------------------
      subroutine addiparam(name, ival)
      include 'head.h'

      character*99 name
      character*99 ipnames(nparam)
      integer ipvals(nparam)
      common /ipn/ ipnames
      common /ipv/ ipvals
      common /ipc/ ipmax
      integer ival, ipmax

      ipmax = ipmax + 1
      if (ipmax .gt. nparam) then
            write(*, *)'exceeded max number of integer::parameters'
            write(*, *)'change nparam in head.h'
            stop
      end if
      call strcopy(name, ipnames(ipmax))
      call tolower(ipnames(ipmax))
      ipvals(ipmax) = ival

      return
      end
!---------------------------------------------------------------------
      subroutine adddparam(name, dval)
      include 'head.h'

      character*99 name
      character*99 dpnames(nparam)
      real*8 dpvals(nparam)
      real*8 dval
      common /dpn/ dpnames
      common /dpv/ dpvals
      common /dpc/ idpmax
      integer idpmax

      idpmax = idpmax + 1
      if (idpmax .gt. nparam) then
            write(*, *)'exceeded max number of double parameters'
            write(*, *)'change nparam in head.h'
            stop
      end if
      call strcopy(name, dpnames(idpmax))
      call tolower(dpnames(idpmax))
      dpvals(idpmax) = dval

      return
      end
!---------------------------------------------------------------------
!- copy string from origin to goal
!---------------------------------------------------------------------
      subroutine strcopy (origin, goal)
      character*99 origin, goal
      integer ilen, i

      ilen= len_trim(origin)

      do i = 1, ilen
         goal(i:i) = origin(i:i)
      end do

      do i = ilen+1, 99
         goal(i:i) = ' '
      end do

      return
      end
!---------------------------------------------------------------------
!- setup all parameters
!---------------------------------------------------------------------
      subroutine setparameters()
      include 'head.h'

      character*99 name
      real*8 dparam
      integer iparam

      name = 'istart'
      istart = iparam(name, 0, 1)

      name = 'iforce'
      iforce = iparam(name, 1, 1)

      name = 'irotate'
      irotate = iparam(name, 1, 1)

      name = 'ihyper'
      ihyper = iparam(name, 1, 1)

      name = 'ihpower'
      ihpower = iparam(name, 8, 1)

      name = 'begin'
      begin = iparam(name, 1, 1)

      name = 'tint'
      tint = iparam(name, 100, 1)

      name = 'tmeas'
      tmeas = iparam(name, 10, 1)

      name = 'ncheck'
      ncheck = iparam(name, 200, 1)

      name = 'scheck'
      scheck = iparam(name, 5, 1)

      name = 'dt'
      dt = dparam(name, 0.001d0, 1)

      name = 'cfl'
      cfl = dparam(name, 0.9d0, 1)

      name = 'romega'
      romega = dparam(name, 4.d0, 1)

      name = 'rnu'
      rnu = dparam(name, 0.01d0, 1)

      name = 'epsf'
      epsf = dparam(name, 1.d0, 1)

      name = 'fhigh'
      fhigh = dparam(name, 14.d0, 1)

      name = 'fpeak'
      fpeak = dparam(name, 12.d0, 1)

      name = 'flow'
      flow = dparam(name, 10.d0, 1)

      name = 'ralpha'
      ralpha = dparam(name, 0.05d0, 1)

      name = 'ipositive'
      ipositive = iparam(name, 0, 1)

      name = 'iscalar'
      iscalar = iparam(name, 0, 1)

      name = 'ihypo'
      ihypo = iparam(name, 1, 1)

      name = 'ihopower'
      ihopower = iparam(name, 2, 1)

      return
      end
!---------------------------------------------------------------------
      function iparam(name, idefval, iuse)
      include 'head.h'

      integer iparam
      character*99 name, tname
      character*99 ipnames(nparam)
      integer ipvals(nparam)
      common /ipn/ ipnames
      common /ipv/ ipvals
      common /ipc/ ipmax
      integer i, ipmax, iuse, idefval, ic, nstrcmp, ilen

      call strcopy(name, tname)
      call tolower(tname)

      do i = 1, ipmax
         ic = nstrcmp(tname, ipnames(i))
         if (ic .eq. 1) then
             iparam = ipvals(i)
             return
         end if
      end do

      if (iuse .eq. 0) then
         iparam = idefval
         return
      else
         write(*, *)'unable to find integer::parameter:'
         ilen= len_trim(tname)
         do i = ilen+1, 99
            tname(i:i) = ' '
         end do
         write(*, *) tname
         stop
      end if

      return
      end function iparam
!---------------------------------------------------------------------
      function dparam(name, defval, iuse)
      include 'head.h'

      real*8 dparam
      character*99 name, tname
      character*99 dpnames(nparam)
      real*8 dpvals(nparam), defval
      common /dpn/ dpnames
      common /dpv/ dpvals
      common /dpc/ idpmax
      integer i, ic, iuse, ilen, idpmax
      integer nstrcmp

      call strcopy(name, tname)
      call tolower(tname)

      do i = 1, idpmax
         ic = nstrcmp(tname, dpnames(i))
         if (ic .eq. 1) then
            dparam = dpvals(i)
            return
         end if
      end do

      if (iuse .eq. 0) then
         dparam = defval
         return
      else
         write(*, *)'unable to find double parameter:'
         ilen= len_trim(tname)
         do i = ilen+1, 99
            tname(i:i) = ' '
         end do
         write(*, *) tname
         stop
      end if

      return
      end function dparam
!---------------------------------------------------------------------
      function nstrcmp(name1, name2)
      integer nstrcmp
      character*99 name1, name2, tname1, tname2
      integer ilen1, ilen2, i

      call strcopy(name1, tname1)
      call strcopy(name2, tname2)
      ilen1 = len_trim(tname1)
      ilen2 = len_trim(tname2)

      if (ilen1 .ne. ilen2) then
         nstrcmp = 0
         return
      end if

      call tolower(tname1)
      call tolower(tname2)

      do i = 1, ilen1
         if (tname1(i:i) .ne. tname2(i:i)) then
            nstrcmp = 0
            return
         end if
      end do

      nstrcmp = 1

      return
      end
!---------------------------------------------------------------------
!- read (u, v, w) from the file named filename (binary reader)
!---------------------------------------------------------------------
      subroutine velocity_b_read(u, filename)
      include 'head.h'

      real*8 u(nx, ny, nz, 3)
      character*99 filename

      real*8 tempu(3*nx*ny*nz+1)
      common /vectortemp2/ tempu

      integer ix, iy, iz

      call read8(tempu, 3*nx*ny*nz+1, filename)

      time = tempu(1)

      do iz = 1, nz
      do iy = 1, ny
      do ix = 1, nx
         u(ix,iy,iz,1) = tempu(1+3*(ix-1+nx*(iy-1+ny*(iz-1)))+1)
         u(ix,iy,iz,2) = tempu(2+3*(ix-1+nx*(iy-1+ny*(iz-1)))+1)
         u(ix,iy,iz,3) = tempu(3+3*(ix-1+nx*(iy-1+ny*(iz-1)))+1)
      end do
      end do
      end do

      call rc_3d(u)

      return
      end
!---------------------------------------------------------------------
!- read real(kind=8)::data from a binary file
!---------------------------------------------------------------------
      subroutine read8(u, ptot, filename)
      integer ptot
      real*8 u(ptot)
      character*99 filename

      integer i

!- for Intel's Fortran 90 compiler (2004 or newer)
!- (based on 4Byte-Real unit and little endian format)
      open(99,file=filename,access='direct',recl=2*ptot,
     &form='unformatted',convert='big_endian')

!- for g77 compiler, IFC (2002 or older)
!- (based on 1Byte unit and little endian format(unsolved))
!      open(99,file=filename,access='direct',recl=8*ptot,
!     &form='unformatted')
!- for SGI's compiler (base on 1Byte Unit and big endian format)
!      open(99,file=filename,access='direct',recl=8*ptot,
!     &form='unformatted')

      read(99,rec=1) (u(i),i=1,ptot)

      close(99)

      return
      end
!---------------------------------------------------------------------
!- create a string by rule, str1 = str1+'.'+str2, without check the
!- availability
!---------------------------------------------------------------------
      subroutine cdotappend(str1, str2)
      character*99 str1, str2
      integer ilen1, ilen2

      ilen1 = len_trim(str1)
      ilen2 = len_trim(str2)

      str1(ilen1+1:ilen1+1) = '.'
      str1(ilen1+2:ilen1+ilen2+2) = str2(1:ilen2)

      return
      end
!---------------------------------------------------------------------
!- save real(kind=8)::data to a binary file
!---------------------------------------------------------------------
      subroutine save8(u, ptot, filename)
      integer ptot
      real*8 u(ptot)
      character*99 filename

      integer i

!- for Intel's Fortran 90 compiler (2004 or newer)
!- (based on 4Byte-Real unit and little endian format)
      open(99,file=filename,access='direct',recl=2*ptot,
     &form='unformatted',convert='big_endian')

!- for g77 compiler, IFC (2002 or older)
!- (based on 1Byte unit and little endian format(unsolved))
!      open(99,file=filename,access='direct',recl=8*ptot,
!     &form='unformatted')
!- for SGI's compiler (base on 1Byte Unit and big endian format)
!      open(99,file=filename,access='direct',recl=8*ptot,
!     &form='unformatted')

      write(99, rec = 1) (u(i), i = 1, ptot)

      close(99)

      return
      end
!---------------------------------------------------------------------
!- create a empty file
!---------------------------------------------------------------------
      subroutine empty_file(filename)
      character*99 filename

      open(99,file=filename,status='replace',form='formatted')

      close(99)

      return
      end
!---------------------------------------------------------------------
! output 3D and 2D spectrum of velocity field
!---------------------------------------------------------------------
      subroutine spectrum_u(u, filename, ioutput)
      include 'head.h'

      complex*16 u(nx/2, ny, nz, 3)
      character*99 filename
      integer ioutput

      real*8 spec(smax, 3)
      common /spectrumfield/ spec

!---- arrays of wavenumber ----
      real*8 k(nx/2, ny, nz, 3)
      common /wavenumber/ k

!- array for the hyper and hypo viscosity
      real*8 hyper(nx/2, ny, nz), specmax
      common /hyperblock/ hyper, specmax
      real*8 hypo(nx/2, ny, nz), specmin
      common /hypoblock/ hypo, specmin

      integer istep
      common /steps/ istep

      real*8 epsin
      common /dissrate/ epsin

      integer ix, iy, iz, i, kk, ikk
      real*8 factor, ww, val, sample, tspecmax, tspecmin
      real*8 rk, l1, l2, kmax, rk_l, rk_h, k2d

      real*8 ispec(smax, 2)
      common /ispectrumfield/ ispec

      integer ikpoint(nx/2, ny, nz)
      common /iidwavenumber/ ikpoint

      integer kpoint(nx/2, ny, nz)
      common /idwavenumber/ kpoint

      real*8 spectrum2d(smax)
      
      kmax = dble(min(nx,min(ny,nz)))/3.0d0

      rk_l = kmax-1.0d0
      rk_h = kmax+1.0d0

      factor = 0.125d0

      do i = 1, smax
         spec(i, 2) = 0.0d0
         spec(i, 3) = 0.0d0
         ispec(i, 2) = 0.0d0
         spectrum2d(i) = 0.0d0
      end do

      tspecmax = 0.0d0
      tspecmin = 0.0d0
      if (ihyper .eq. 1) tspecmax = specmax
      if (ihypo .eq. 1) tspecmin = specmin

      specmax = 0.0d0
      sample = 0.0d0
      do i = 1, 3

      do iz = 1, nz
      do iy = 1, ny
      do ix = 1, nx/2
         rk = k(ix,iy,iz,1)**2+k(ix,iy,iz,2)**2+k(ix,iy,iz,3)**2
         rk = dsqrt(rk)
         if (rk .le. (kmax+10.0d0)) then
            kk = kpoint(ix, iy, iz)
            ikk = ikpoint(ix, iy, iz)

            ww = 2.0d0 * factor
            if (ix .eq. 1) ww = factor

            val = ww*u(ix, iy, iz, i)*dconjg(u(ix, iy, iz, i))
            spec(kk, 2) = spec(kk, 2)+val
            ispec(ikk, 2) = ispec(ikk, 2)+val
            spec(ikk, 3) = spec(ikk, 3) + 2.d0*rk*rk*val*(rnu +
     &tspecmax*hyper(ix, iy, iz)+tspecmin*hypo(ix, iy, iz))
            if (iz .eq. 1) spectrum2d(ikk) = spectrum2d(ikk)+val

!- calculate the energy at ultraviolet range for hyperviscosity
            if (rk .ge. rk_l .and. rk .le. rk_h) then
               sample = sample + 2.d0*ww
               specmax = specmax + val
            end if
         end if

      end do
      end do
      end do

      end do

      specmax = specmax*4.0d0*pi*rk_h**2/sample
      specmax = dsqrt(specmax)*2.5d0
!      specmax = dsqrt(specmax)*5.0d0
! same as the way above to get specmax, but little smaller
!      ikk = ikpoint(kmax, 1, 1)
!      specmax = dsqrt(ispec(ikk+1, 2))*2.5d0

      specmin = dsqrt(spec(2, 2))

      etot(1) = 0.d0
      k2d = 0.0d0
      dtot = 0.d0
      l1 = 0.0d0
      l2 = 0.0d0

      do i = 1, smax-1
         spec(i, 2) = spec(i, 2)/(spec(i+1, 1)-spec(i, 1))
         spec(i, 3) = spec(i, 3)/(spec(i+1, 1)-spec(i, 1))
         ispec(i, 2) = ispec(i, 2)/(ispec(i+1, 1)-ispec(i, 1))
         spectrum2d(i) = spectrum2d(i)/(ispec(i+1, 1)-ispec(i, 1))
      end do

      do i = 1, smax-1
         etot(1) = etot(1) + ispec(i, 2)*(ispec(i+1, 1)-ispec(i, 1))
         k2d = k2d + spectrum2d(i)*(ispec(i+1, 1)-ispec(i, 1))
         dtot = dtot + spec(i, 3)*(ispec(i+1, 1)-ispec(i, 1))
      end do

      do i = 2, smax-1
         l1 = l1 + ispec(i, 2)/ispec(i, 1)*(ispec(i+1, 1)-ispec(i, 1))
         l2 = l2 + ispec(i, 2)*ispec(i, 1)*(ispec(i+1, 1)-ispec(i, 1))
      end do

      l = etot(1)**(3.0d0/2.0d0)/dtot
      eta = (rnu**3.0d0/dtot)**(1.0d0/4.0d0)
      rel = (etot(1)**2.0d0/dtot/rnu)
      re = (20.0d0/3.0d0*rel)**(1.0d0/2.0d0)
      v = (etot(1)*2.0d0/3.0d0)**(1.0d0/2.0d0)
      tau_t = etot(1)/dtot

      ros(1) = v*fpeak/(2.0d0*pi)/(2.0d0*romega)
      ros(2) = dtot/(etot(1)*2.0d0*romega)

      l1 = l1/(v*v)
      l2 = v*v/l2
      ros(3) = v/(l1*2.0d0*romega)
      ros(4) = v/(l2*2.0d0*romega)

      if (ioutput .eq. 1) then

      open(99, file = filename)

      write(99,*) '#k E D E2d'
      write(99,35)time,istep
      write(99,37)etot(1),dtot,epsin,l,eta,rel,
     &re,v,tau_t,dsqrt(rnu/dtot)

      do i = 1, smax
      if (dabs(spec(i, 2)) .gt. 1.d-20 .and.
     &spec(i, 1) .le. (kmax+2)) then
!      if (dabs(spec(i, 2)) .gt. 1.d-20) then
      write(99, 36) ispec(i,1),ispec(i,2),spec(i,3),spectrum2d(i)
      end if
      end do

35    format(1x, '# 3d spectrum at time = ', e14.7, ' (', i6, ')')
36    format(1x, 1e20.11,'  ',1e20.11,'  ',1e20.11,'  ',1e20.11)
37    format(1x, '# k = ', 1e15.6, ' epsilon = ', 1e15.6,
     &':', 1e15.6, ' l = ', 1e15.6, ' eta = ', 1e15.6, ' re_l = ',
     &1e15.6, ' re_lambda = ', 1e15.6, ' u = ', 1e15.6,
     &' tau = ', 1e15.6, ':', 1e15.6)
      close(99)

      call evolution(time,istep,etot(1),etot(5),tau_t,re,eta,
     &1.0d0/(2.0d0*romega*tau_t),etot(3)/(2.0d0*romega),
     &etot(4)/(2.0d0*romega),etot(3)/(2.0d0*romega*dsqrt(15.0d0)),
     &k2d/etot(1))

      call evolution_spectrums()

      end if

      end
!---------------------------------------------------------------------
!- output the (time, step, total energy, ...)
!---------------------------------------------------------------------
      subroutine evolution(t,s,k,esgs1,tau_t,re,eta,rosL,
     &roso,roso3,rosla,k2r)

      real*8 t,k,esgs1,tau_t,re,eta,rosL,roso,roso3,rosla,k2r
      integer s

      character*99 filename

      filename = 'evolution.dat'

      open(99, file = filename, status = 'old', form = 'formatted',
     &access = 'append')
!- for some compilers
!      open(99, file = filename, status = 'old', form = 'formatted',
!     &position = 'append')

      write(99, 10) 
     &t,s,k,esgs1,tau_t,re,eta,rosL,roso,roso3,rosla,k2r
10    format(1x,1e18.9,' ',i6,' ',1e18.9,' ',1e18.9,' ',1e18.9,' ',
     &1e18.9,' ',1e18.9,' ',1e18.9,' ',1e18.9,' ',1e18.9,' ',1e18.9,
     &' ',1e18.9)

      close(99)

      return
      end
!---------------------------------------------------------------------
!- output the (time, transfer, source, pi)
!---------------------------------------------------------------------
      subroutine evolution_spectrums()
      include 'head.h'

      real*8 spec(smax, 3)
      common /spectrumfield/ spec

      integer i
      character*99 filename

10    format(1x, 1e18.9, 23e18.9)

      filename = 'evolution_spec.dat'
      open(99, file = filename, status = 'old', form = 'formatted',
     &access = 'append')
!- for some compilers
!      open(99, file = filename, status = 'old', form = 'formatted',
!     &position = 'append')

      write(99, 10) time, (spec(i, 2), i = 2, 24)
      close(99)

      return
      end
!---------------------------------------------------------------------
!- output (u, v, w) to the file named filename (binary output)
!---------------------------------------------------------------------
      subroutine velocity_b_output(u, filename)
      include 'head.h'

      real*8 u(nx, ny, nz, 3)
      character*99 filename

      real*8 tempu(nx, ny, nz, 3)
      common /vectortemp1/ tempu

      real*8 tempu2(3*nx*ny*nz+1)
      common /vectortemp2/ tempu2

      integer ix, iy, iz

      call arraycopy_3d(tempu, u)
      call cr_3d(tempu)

      tempu2(1) = time

      do iz = 1, nz
      do iy = 1, ny
      do ix = 1, nx
         tempu2(1+3*(ix-1+nx*(iy-1+ny*(iz-1)))+1) = tempu(ix,iy,iz,1)
         tempu2(2+3*(ix-1+nx*(iy-1+ny*(iz-1)))+1) = tempu(ix,iy,iz,2)
         tempu2(3+3*(ix-1+nx*(iy-1+ny*(iz-1)))+1) = tempu(ix,iy,iz,3)
      end do
      end do
      end do

      call save8(tempu2, 3*nx*ny*nz+1, filename)

      return
      end
!---------------------------------------------------------------------
      subroutine power3d(var, filename)
      include 'head.h'

      complex*16 var(nx/2, ny, nz, 3)
      character*99 filename

      real*8 spec(smax, 3)
      common /spectrumfield/ spec

      real*8 power(smax)

!---- arrays of wavenumber ----
      real*8 k(nx/2, ny, nz, 3)
      common /wavenumber/ k

      integer ix, iy, iz, i, kk
      real*8 factor, ww, val
      real*8 rk, kmax

      integer kpoint(nx/2, ny, nz)
      common /idwavenumber/ kpoint

      kmax = dble(min(nx,min(ny,nz)))/3.0d0

      factor = 0.5d0

      do i = 1, smax
         power(i) = 0.0d0
      end do

      do i = 1, 3

      do iz = 1, nz
      do iy = 1, ny
      do ix = 1, nx/2
         rk = k(ix,iy,iz,1)**2+k(ix,iy,iz,2)**2+k(ix,iy,iz,3)**2
         rk = dsqrt(rk)
         if (rk .le. (kmax+10.0d0)) then
            kk = kpoint(ix, iy, iz)

            ww = 2.0d0 * factor
            if (ix .eq. 1) ww = factor

            val = ww*var(ix, iy, iz, i)*dconjg(var(ix, iy, iz, i))
            power(kk) = power(kk)+val

         end if

      end do
      end do
      end do

      end do

      do i = 1, smax-1
         power(i) = power(i)/(spec(i+1, 1)-spec(i, 1))
      end do

      open(99, file = filename)

      write(99,*) '#k power'

      do i = 1, smax
      if (dabs(power(i)) .gt. 1.d-20 .and.
     &spec(i, 1) .le. (kmax+2)) then
!      if (dabs(spec(i, 2)) .gt. 1.d-20) then
      write(99, 36) spec(i, 1), power(i)
      end if
      end do

36    format(1x, 1e20.11, '  ', 1e20.11)

      end
!---------------------------------------------------------------------
! copy integer to string like "00001"
!---------------------------------------------------------------------
      subroutine zerofill(str,num,length)
	
      character*99 str
      integer num, length

      integer ilen,d,k,kmd

      write(str,100) num
100   format(i7)

      call stripspace(str)
      ilen	= len_trim(str)
      if (ilen < length) then
         d	= length-ilen
         do k = length+1,d+1,-1
            kmd	= k-d
            str(k:k)	= str(kmd:kmd)
         end do
         do k = d,1,-1
            str(k:k)	= '0'
         end do			
      end if

      end
!---------------------------------------------------------------------
!- save real(kind=4)::data to a binary file
!---------------------------------------------------------------------
      subroutine save4(u, ptot, filename)
      integer ptot
      real*4 u(ptot)
      character*99 filename

      integer i

!- for Intel's Fortran 90 compiler (2004 or newer)
!- (based on 4Byte-Real unit and little endian format)
      open(99,file=filename,access='direct',recl=ptot,
     &form='unformatted',convert='big_endian')

!- for g77 compiler, IFC (2002 or older)
!- (based on 1Byte unit and little endian format(unsolved))
!      open(99,file=filename,access='direct',recl=4*ptot,
!     &form='unformatted')
!- for SGI's compiler (base on 1Byte Unit and big endian format)
!      open(99,file=filename,access='direct',recl=4*ptot,
!     &form='unformatted')

      write(99, rec = 1) (u(i), i = 1, ptot)

      close(99)

      return
      end
!---------------------------------------------------------------------
!- read real(kind=4)::data from a binary file
!---------------------------------------------------------------------
      subroutine read4(u, ptot, filename)
      integer ptot
      real*4 u(ptot)
      character*99 filename

      integer i

!- for Intel's Fortran 90 compiler (2004 or newer)
!- (based on 4Byte-Real unit and little endian format)
      open(99,file=filename,access='direct',recl=ptot,
     &form='unformatted',convert='big_endian')

!- for g77 compiler, IFC (2002 or older)
!- (based on 1Byte unit and little endian format(unsolved))
!      open(99,file=filename,access='direct',recl=4*ptot,
!     &form='unformatted')
!- for SGI's compiler (base on 1Byte Unit and big endian format)
!      open(99,file=filename,access='direct',recl=4*ptot,
!     &form='unformatted')

      read(99,rec=1) (u(i),i=1,ptot)

      close(99)

      return
      end
!---------------------------------------------------------------------
!- read (u, v, w) from the file named filename (binary reader)
!---------------------------------------------------------------------
      subroutine velocity_b4_read(u, filename)
      include 'head.h'

      real*8 u(nx, ny, nz, 3)
      character*99 filename

      real*4 tmpu(3*nx*ny*nz+1)
      common /vectortemp2/ tmpu

      integer ix, iy, iz

      call read4(tmpu, 3*nx*ny*nz+1, filename)

      time = dble(tmpu(1))

      do iz = 1, nz
      do iy = 1, ny
      do ix = 1, nx
         u(ix,iy,iz,1)= dble(tmpu(1+3*(ix-1+nx*(iy-1+ny*(iz-1)))+1))
         u(ix,iy,iz,2)= dble(tmpu(2+3*(ix-1+nx*(iy-1+ny*(iz-1)))+1))
         u(ix,iy,iz,3)= dble(tmpu(3+3*(ix-1+nx*(iy-1+ny*(iz-1)))+1))
      end do
      end do
      end do

      call rc_3d(u)

      return
      end
!---------------------------------------------------------------------
!- output (u, v, w) to the file named filename (binary output)
!---------------------------------------------------------------------
      subroutine velocity_b4_output(u, filename)
      include 'head.h'

      real*8 u(nx, ny, nz, 3)
      character*99 filename

      real*8 tempu(nx, ny, nz, 3)
      common /vectortemp1/ tempu

      real*4 tmpu(3*nx*ny*nz+1)
      common /vectortemp2/ tmpu

      integer ix, iy, iz

      call arraycopy_3d(tempu, u)
      call cr_3d(tempu)

      tmpu(1) = real(time)

      do iz = 1, nz
      do iy = 1, ny
      do ix = 1, nx
      	
      tmpu(1+3*(ix-1+nx*(iy-1+ny*(iz-1)))+1)=real(tempu(ix,iy,iz,1))
      tmpu(2+3*(ix-1+nx*(iy-1+ny*(iz-1)))+1)=real(tempu(ix,iy,iz,2))
      tmpu(3+3*(ix-1+nx*(iy-1+ny*(iz-1)))+1)=real(tempu(ix,iy,iz,3))

      end do
      end do
      end do

      call save4(tmpu, 3*nx*ny*nz+1, filename)

      return
      end
!---------------------------------------------------------------------
!- output single structured xyz file for Plot3D, 3D version
!---------------------------------------------------------------------
      subroutine plot3d_xyz_3d(x,y,z,nx,ny,nz,filename)
      integer nx, ny, nz
      real*4 x(nx, ny, nz), y(nx, ny, nz), z(nx, ny, nz)
      character*99 filename
      
      integer ix, iy, iz
      
      open(99,file=filename,form='binary',status='replace',
     &convert='big_endian')
      write(99) nx, ny, nz
      write(99) (((x(ix,iy,iz),ix=1,nx),iy=1,ny),iz=1,nz),
     &(((y(ix,iy,iz),ix=1,nx),iy=1,ny),iz=1,nz),
     &(((z(ix,iy,iz),ix=1,nx),iy=1,ny),iz=1,nz)
      close(99)

      return
      end
!---------------------------------------------------------------------
!- output q file of single structured grid for Plot3D, 3D version
!---------------------------------------------------------------------
      subroutine plot3d_q_3d(q,fsmach,alpha,re,time,nx,ny,nz,filename)
      integer nx, ny, nz
      real*4 fsmach, alpha, re, time
      real*4 q(nx, ny, nz, 5)
      character*99 filename

      integer ix, iy, iz, l
      
      open(99,file=filename,form='binary',status='replace',
     &convert='big_endian')
      write(99) nx, ny, nz
      write(99) fsmach, alpha, re, time
      write(99) ((((q(ix,iy,iz,l),ix=1,nx),iy=1,ny),iz=1,nz),l=1,5)
      close(99)
      
      return
      end
!---------------------------------------------------------------------
!- output single structured xyz file for Plot3D, 2D version
!---------------------------------------------------------------------
      subroutine plot3d_xyz_2d(x,y,nx,ny,filename)
      integer nx, ny
      real*4 x(nx, ny), y(nx, ny)
      character*99 filename
      
      integer ix, iy
      
      open(99,file=filename,form='binary',status='replace',
     &convert='big_endian')
      write(99) nx, ny
      write(99) ((x(ix,iy),ix=1,nx),iy=1,ny),
     &((y(ix,iy),ix=1,nx),iy=1,ny)
      close(99)

      return
      end
!---------------------------------------------------------------------
!- output q file of single structured grid for Plot3D, 2D version
!---------------------------------------------------------------------
      subroutine plot3d_q_2d(q,fsmach,alpha,re,time,nx,ny,filename)
      integer nx, ny
      real*4 fsmach, alpha, re, time
      real*4 q(nx, ny, 4)
      character*99 filename
      
      integer ix, iy, l

      open(99,file=filename,form='binary',status='replace',
     &convert='big_endian')
      write(99) nx, ny
      write(99) fsmach, alpha, re, time
      write(99) (((q(ix,iy,l),ix=1,nx),iy=1,ny),l=1,4)
      close(99)
      
      return
      end
!---------------------------------------------------------------------
!- output xyz file, 3D version
!---------------------------------------------------------------------
      subroutine xyz_3d_output(filename)
      include 'head.h'
      
      character*99 filename
      
      real*4 x(nx, ny, nz), y(nx, ny, nz), z(nx, ny, nz)
      common /vectortemp2/ x, y, z
      
      real*4 dx, dy, dz
      integer ix, iy, iz
      
      dx = 2.0*real(pi)/real(nx)
      dy = 2.0*real(pi)/real(ny)
      dz = 2.0*real(pi)/real(nz)
      do iz = 1, nz
         do iy = 1, ny
            do ix = 1, nx
            x(ix,iy,iz) = -real(pi)+dx*real(ix-1)
            y(ix,iy,iz) = -real(pi)+dy*real(iy-1)
            z(ix,iy,iz) = -real(pi)+dz*real(iz-1)
            end do
         end do
      end do

      call plot3d_xyz_3d(x,y,z,nx,ny,nz,filename)
                  
      end
!---------------------------------------------------------------------
!- output xyz file, 2D version
!---------------------------------------------------------------------
      subroutine xyz_2d_output(filename)
      include 'head.h'
      
      character*99 filename
      
      real*4 x(nx, ny, 1), y(nx, ny, 1), z(nx, ny, 1)
      common /vectortemp2/ x, y, z
      
      real*4 dx, dy
      integer ix, iy
      
      dx = 2.0*real(pi)/real(nx)
      dy = 2.0*real(pi)/real(ny)
      do iy = 1, ny
         do ix = 1, nx
         x(ix,iy, 1) = -real(pi)+dx*real(ix-1)
         y(ix,iy, 1) = -real(pi)+dy*real(iy-1)
         z(ix,iy, 1) = 0.0
         end do
      end do
      call plot3d_xyz_3d(x,y,z,nx,ny,1,filename)
                  
      end
!---------------------------------------------------------------------
!- output q file (including k, u, v, w and vorticity), 3D version
!---------------------------------------------------------------------
      subroutine q_3d_output(u,filename)
      include 'head.h'

      real*8 u(nx, ny, nz, 3)
      character*99 filename

      real*8 omega(nx, ny, nz, 3)
      common /vectortemp1/ omega

      real*8 ur(nx, ny, nz, 3)
      common /vectortemp3/ ur
      
      real*4 q(nx, ny, nz, 5)
      common /vectortemp2/ q
      real*4 fsmach, alpha
      
      integer ix, iy, iz
      
      call arraycopy_3d(ur, u)
      call curl(omega, ur)
      call cr_3d(omega)
      call cr_3d(ur)
      
      do iz = 1, nz
      do iy = 1, ny
      do ix = 1, nx
         q(ix,iy,iz,2) = real(ur(ix,iy,iz,1))
         q(ix,iy,iz,3) = real(ur(ix,iy,iz,2))
         q(ix,iy,iz,4) = real(ur(ix,iy,iz,3))
         q(ix,iy,iz,1) = (q(ix,iy,iz,1)**2+q(ix,iy,iz,2)**2+
     &q(ix,iy,iz,3)**2)/2.0
         q(ix,iy,iz,5) = (omega(ix,iy,iz,1)**2+omega(ix,iy,iz,2)**2+
     &omega(ix,iy,iz,3))/2.0
      end do
      end do
      end do
      
      fsmach = 0.0
      alpha = 0.0
      call plot3d_q_3d(q,fsmach,alpha,real(re),real(time),
     &nx,ny,nz,filename)
      
      end
!---------------------------------------------------------------------
!- output q file (including k, u, v, w and vorticity), 2D version
!---------------------------------------------------------------------
      subroutine q_2d_output(u, filename)
      include 'head.h'

      real*8 u(nx, ny, nz, 3)
      character*99 filename

      real*8 omega(nx, ny, nz, 3)
      common /vectortemp1/ omega

      real*8 ur(nx, ny, nz, 3)
      common /vectortemp3/ ur
      
      real*4 q(nx, ny, 1, 5)
      common /vectortemp2/ q
      real*4 fsmach, alpha
      
      integer ix, iy, iz
      
      call arraycopy_3d(ur, u)
      call curl(omega, ur)
      call cr_3d(omega)
      call cr_3d(ur)
      
      do iy = 1, ny
      do ix = 1, nx
         q(ix,iy,1,1) = 0.0
         q(ix,iy,1,2) = 0.0
         q(ix,iy,1,3) = 0.0
         q(ix,iy,1,4) = 0.0
         q(ix,iy,1,5) = 0.0
         do iz = 1, nz
            q(ix,iy,1,1) = q(ix,iy,1,1)+(ur(ix,iy,iz,1)**2+
     &ur(ix,iy,iz,2)**2+ur(ix,iy,iz,3)**2)/2.0
            q(ix,iy,1,2) = q(ix,iy,1,2)+ur(ix,iy,iz,1)
            q(ix,iy,1,3) = q(ix,iy,1,3)+ur(ix,iy,iz,2)
            q(ix,iy,1,4) = q(ix,iy,1,4)+ur(ix,iy,iz,3)
            q(ix,iy,1,5) = q(ix,iy,1,5)+(omega(ix,iy,iz,1)**2+
     &omega(ix,iy,iz,2)**2+omega(ix,iy,iz,3))/2.0
         end do
         q(ix,iy,1,1) = q(ix,iy,1,1)/real(nz)
         q(ix,iy,1,2) = q(ix,iy,1,2)/real(nz)
         q(ix,iy,1,3) = q(ix,iy,1,3)/real(nz)
         q(ix,iy,1,4) = q(ix,iy,1,4)/real(nz)
         q(ix,iy,1,5) = q(ix,iy,1,5)/real(nz)
      end do
      end do
      
      fsmach = 0.0
      alpha = 0.0
      call plot3d_q_3d(q,fsmach,alpha,real(re),real(time),
     &nx,ny,1,filename)
      
      end
!---------------------------------------------------------------------
