!---------------------------------------------------------------------
!- setup viscosity term for hyper and hypo - viscosity
!---------------------------------------------------------------------
      subroutine setupvisc(hyper, hypo)
      include 'head.h'

      real*8 hyper(nx/2, ny, nz), hypo(nx/2, ny, nz)

!---- arrays of wavenumber ----
      real*8 k(nx/2, ny, nz, 3)
      common /wavenumber/ k

      real*8 rk, kmax, kmin

      integer ix, iy, iz, ih2, iu2

      kmin = 1.0d0
      kmax = dble(min(nx,min(ny,nz)))/3.0d0

      if (ihyper .eq. 1) then

      ih2 = 2*ihpower-2
      do iz = 1, nz
      do iy = 1, ny
      do ix = 1, nx/2

         rk = k(ix,iy,iz,1)**2+k(ix,iy,iz,2)**2+k(ix,iy,iz,3)**2
         rk = dsqrt(rk)
!      if (rk .le. kmax) then
         hyper(ix,iy,iz) = 2.5d0*dsqrt(1.0d0/kmax)*(rk/kmax)**ih2
!      else
!      hyper(ix, iy, iz) = 0.0d0
!      end if

      end do
      end do
      end do

      else

      do iz = 1, nz
      do iy = 1, ny
      do ix = 1, nx/2
         hyper(ix,iy,iz) = 0.0d0
      end do
      end do
      end do

      end if

      if (ihypo .eq. 1) then

      iu2 = 2*ihopower+2
      do iz = 1, nz
      do iy = 1, ny
      do ix = 1, nx/2

         rk = k(ix,iy,iz,1)**2+k(ix,iy,iz,2)**2+k(ix,iy,iz,3)**2
         rk = dsqrt(rk)
         hypo(ix,iy,iz) = 1.0d0*dsqrt(1.0d0/kmin)*(kmin/rk)**iu2

      end do
      end do
      end do

      hypo(1,1,1) = 0.0d0

      else

      do iz = 1, nz
      do iy = 1, ny
      do ix = 1, nx/2
         hypo(ix,iy,iz) = 0.0d0
      end do
      end do
      end do

      end if

      return
      end
!---------------------------------------------------------------------
      function modstep(a, b)
      integer modstep
      integer a, b

      if (mod(a-1, b) .eq. 0) then
         modstep = 1
      else
         modstep = 0
      end if

      return
      end
!---------------------------------------------------------------------
!- initialize the velocity field
!- if it is the first time to run, code will create a random velocity
!- field if not, code will load the previous saved field
!---------------------------------------------------------------------
      subroutine init(u)
      include 'head.h'

      complex*16 u(nx/2, ny, nz, 3)

      real*8 en(nmax)
      integer ipope
      common /pope/ ipope

      real*8 spec(nmax), irforce
      common /spec_rforce/ spec, irforce

      character*99 name

!---- arrays of wavenumber ----
      real*8 k(nx/2, ny, nz, 3)
      common /wavenumber/ k

      integer ix, iy, iz, i, kk

      real*8 realpart(3), imagepart(3), anorm, k2

      ipope = 0
      irforce = 0.0d0

      if (istart .eq. 1 .or. istart .eq. 2) then
!- reload the previsou saved velocity field
      write(*,*)'loading previous velocity profile...'
         if (mode .eq. 0 .or. mode .eq. 2) then
         name = 'initial.dat'
         call velocity_b_read(u, name)
         else if (mode .eq. 3) then
         name = 'initial4.dat'
         call velocity_b4_read(u, name)
         end if
            	
      else if (istart .eq. 0) then
!- create a new field
      write(*,*)'creating new initial velocity profile...'

      time = 0.0d0

      do i = 1, nmax
            en(i) = 0.0d0
      end do
      en(1) = 0.07147
      en(2) = 0.21425
      en(3) = 0.23865
      en(4) = 0.19223
      en(5) = 0.15619
      en(6) = 0.10572
      en(7) = 0.09513
      en(8) = 0.06951
      en(9) = 0.06164
      en(10) = 0.04781
      en(11) = 0.03739
      en(12) = 0.03213
      en(13) = 0.02737
      en(14) = 0.02168
      en(15) = 0.01896
      en(16) = 0.01545
      en(17) = 0.01407
      en(18) = 0.01068
      en(19) = 0.00841
      en(20) = 0.00512
      en(21) = 0.00228
      en(22) = 8.03214E-4
      do i = 1, nmax
         en(i) = en(i)*0.001
      end do

      do iz = 1, nz
      do iy = 1, ny
      do ix = 1, nx/2

         do i = 1, 3
            realpart(i) = anorm()
            imagepart(i) = anorm()
         end do

         k2 = k(ix,iy,iz,1)**2+k(ix,iy,iz,2)**2+k(ix,iy,iz,3)**2
         kk = int(dsqrt(k2))+1

         do i = 1, 3
            u(ix,iy,iz,i) = dcmplx(realpart(i),imagepart(i))*en(kk)
         end do

      end do
      end do
      end do

      end if

      call reform(u)
      call truncate_3d(u)

      return
      end
!---------------------------------------------------------------------
!- it gives a random number from (-infinite, +infinite) by monte carlo
!- method radius [0, +infinite) satisfies the distribution of
!- exp(-x^2/2)/sqrt(2Pi)
!---------------------------------------------------------------------
      function anorm()
      implicit double precision (a-h, o-z)

      real*8 anorm
      real*8 pi2

      real*8 r1, r2, y

      pi2 = 6.283185307179586476925286766558d0

!- two pseudorandom numbers from the uniform distribution in [0, 1)
!- in fortran 90 library call random_number(r) instead
      call own_random_number(r1)
      call own_random_number(r2)

      y = dsqrt(dabs(2.0d0*dlog(r2+1.0d-9)))
      anorm = y*dcos(pi2*r1)

      return
      end
!---------------------------------------------------------------------
!- give a pseudo-random number in (0,1)
!---------------------------------------------------------------------
      subroutine own_random_number(r)
      real*8 r, zbqlu01

      r = zbqlu01(20051010.0d0)

      return
      end
!---------------------------------------------------------------------
!- check the stable condition
!---------------------------------------------------------------------
      subroutine checkstable(u)
      include 'head.h'

      real*8 u(nx, ny, nz, 3)

      real*8 vv(nx, ny, nz, 3)

      integer ix, iy, iz
      real*8 utmp, umax, dx, umax2, tmp_dt, para_dt

      character*99 name
      real*8 dparam

      call arraycopy_3d(vv, u)

      call cr_3d(vv)

      umax = 0.0d0
      umax2 = 0.0d0
      do iz = 1, nz
      do iy = 1, ny
      do ix = 1, nx

         utmp = dabs(vv(ix, iy, iz, 1))+dabs(vv(ix, iy, iz, 2))
     &+dabs(vv(ix, iy, iz, 3))
         umax = max(utmp, umax)
         umax2 = umax2+vv(ix,iy,iz,1)**2+vv(ix,iy,iz,2)**2
     &+vv(ix,iy,iz,3)**2

      end do
      end do
      end do

      umax2 = umax2/dble(nx*ny*nz)
      etot(5) = umax2/2.0d0
      umax2 = dsqrt(umax2)
      dx = 2.0d0*pi/nx
!      dx = 1.0d0/nx !smaller time step
!      umax = max(0.05d0, umax)
!      umax = max(0.8d0, umax)

      tmp_dt = min(cfl/(umax/dx), cfl/(umax2/dx))

      if (iforce .eq. 0) then
      dt = tmp_dt
      else
!      dt = min(dt, tmp_dt)
      dt = min(1.2d0*dt, tmp_dt)
      end if

!      dt = min(dt, tmp_dt)
!      dt = tmp_dt

      if (irotate .eq. 1) then
            dt = min(0.8d0/romega, dt)
      end if

      name = 'dt'
      para_dt = dparam(name, 0.001, 1)
      dt = min(dt, para_dt)

      return
      end
!---------------------------------------------------------------------
!- update the velocity and scalar by 3rd order runge kutta
!---------------------------------------------------------------------
      subroutine update3(u)
      include 'head.h'

      complex*16 u(nx/2, ny, nz, 3)

      complex*16 t(nx/2, ny, nz, 3)
      common /vectortemp1/ t
      complex*16 up(nx/2, ny, nz, 3)
      common /vectortemp2/ up
      complex*16 tp(nx/2, ny, nz, 3)
      common /vectortemp3/ tp

!---- arrays of wavenumber ----
      real*8 k(nx/2, ny, nz, 3)
      common /wavenumber/ k

!- array for the hyper and hypo viscosity
      real*8 hyper(nx/2, ny, nz), specmax
      common /hyperblock/ hyper, specmax
      real*8 hypo(nx/2, ny, nz), specmin
      common /hypoblock/ hypo, specmin

      real*8 specmaxtmp, specmintmp

      real*8 dt3, dt4, dt3ov4, dt2ov3, k2, temp1, temp2
      integer ix, iy, iz, i

      specmaxtmp = 0.0d0
      specmintmp = 0.0d0
      if (ihyper .eq. 1) specmaxtmp = specmax
      if (ihypo .eq. 1)  specmintmp = specmin

      dt3 = dt/3.0d0
      dt4 = dt/4.0d0
      dt3ov4 = dt*3.0d0/4.0d0
      dt2ov3 = dt*2.0d0/3.0d0

!------------1st step
!- for velocity
      call totalnonlinear(t, u, 1)
      call arraycopy_3d(up, u)

!- project it, make it to be incomressible, add the rotation effects
      call project(t, dt3)
      call project(up, dt3)

      do iz = 1, nz
      do iy = 1, ny
      do ix = 1, nx/2

         k2 = k(ix,iy,iz,1)**2+k(ix,iy,iz,2)**2+k(ix,iy,iz,3)**2
         temp1 = dexp(-(rnu+specmaxtmp*hyper(ix,iy,iz)+
     &specmintmp*hypo(ix, iy, iz))*k2*dt3)

         do i = 1, 3
            up(ix,iy,iz,i) = (up(ix,iy,iz,i)+dt3*t(ix,iy,iz,i))*temp1
         end do

      end do
      end do
      end do

!------------2nd step
!- for velocity
      call totalnonlinear(tp, up, 2)
      call arraycopy_3d(up, u)

!- project it, make it to be incomressible, add the rotation effects
      call project(tp, dt3)
      call project(up, dt2ov3)

      do iz = 1, nz
      do iy = 1, ny
      do ix = 1, nx/2

         k2 = k(ix,iy,iz,1)**2+k(ix,iy,iz,2)**2+k(ix,iy,iz,3)**2
         temp1 = dexp(-(rnu+specmaxtmp*hyper(ix, iy, iz)+
     &specmintmp*hypo(ix, iy, iz))*k2*dt3)
         temp2 = temp1*temp1

         do i = 1, 3
            up(ix,iy,iz,i)=up(ix,iy,iz,i)*temp2+
     &dt2ov3*tp(ix,iy,iz,i)*temp1
         end do

      end do
      end do
      end do

!------------3rd step
!- for velocity
      call totalnonlinear(tp, up, 3)

!- project it, make it to be incomressible, add the rotation effects
      call project(t, dt2ov3)
      call project(tp, dt3)
      call project(u, dt)

      do iz = 1, nz
      do iy = 1, ny
      do ix = 1, nx/2

         k2 = k(ix,iy,iz,1)**2+k(ix,iy,iz,2)**2+k(ix,iy,iz,3)**2
         temp1 = dexp(-(rnu+specmaxtmp*hyper(ix,iy,iz)+
     &specmintmp*hypo(ix,iy,iz))*k2*dt)
         temp2 = dexp(-(rnu+specmaxtmp*hyper(ix,iy,iz)+
     &specmintmp*hypo(ix,iy,iz))*k2*dt3)
         do i = 1, 3
            u(ix,iy,iz,i) = (u(ix,iy,iz,i)+dt4*t(ix,iy,iz,i))*temp1+
     &dt3ov4*tp(ix, iy, iz, i)*temp2
         end do

      end do
      end do
      end do

      return
      end
!---------------------------------------------------------------------
!- projection by helical decomposition
!---------------------------------------------------------------------
      subroutine project(a, ddt)
      include 'head.h'

      complex*16 a(nx/2, ny, nz, 3)
      real*8 ddt

!---- arrays of wavenumber ----
      real*8 k(nx/2, ny, nz, 3)
      common /wavenumber/ k

      real*8 rk2, rk, rkh, freq
      complex*16 em(3), ep(3), bm, bp, ci

      integer ix, iy, iz

      ci = dcmplx(0.0d0, 1.0d0)

      do iz = 1, nz
      do iy = 1, ny
      do ix = 1, nx/2

         rkh = k(ix, iy, iz, 1)**2+k(ix, iy, iz, 2)**2
         rk2 = rkh+k(ix, iy, iz, 3)**2
         rk = dsqrt(rk2)
         rkh = dsqrt(rkh)

         call eigm(k(ix,iy,iz,1),k(ix,iy,iz,2),
     &k(ix,iy,iz,3),rkh,rk,em)

         ep(1) = dconjg(em(1))
         ep(2) = dconjg(em(2))
         ep(3) = dconjg(em(3))

         freq = 2.0d0*romega*dabs(k(ix, iy, iz, 3))/rk
         bm = ep(1)*a(ix,iy,iz,1)+ep(2)*a(ix,iy,iz,2)+
     &ep(3)*a(ix,iy,iz,3)
         bp = em(1)*a(ix,iy,iz,1)+em(2)*a(ix,iy,iz,2)+
     &em(3)*a(ix,iy,iz,3)

         bm = cdexp(ci*freq*ddt)*bm
         bp = cdexp(-ci*freq*ddt)*bp

         a(ix, iy, iz, 1) = bm*em(1)+bp*ep(1)
         a(ix, iy, iz, 2) = bm*em(2)+bp*ep(2)
         a(ix, iy, iz, 3) = bm*em(3)+bp*ep(3)

      end do
      end do
      end do

      a(1, 1, 1, 1) = dcmplx(0.0d0, 0.0d0)
      a(1, 1, 1, 2) = dcmplx(0.0d0, 0.0d0)
      a(1, 1, 1, 3) = dcmplx(0.0d0, 0.0d0)

      return
      end
!---------------------------------------------------------------------
!- get eigenvectors for project
!---------------------------------------------------------------------
      subroutine eigm(kx, ky, kz, kh, k, em)
      real*8 kx, ky, kz, kh, k
      complex*16 em(3)

      complex*16 ci
      real*8 dsq2, fac

      ci = dcmplx(0.0d0, 1.0d0)
      dsq2 = dsqrt(2.0d0)

      if (kh < 0.000001d0) then
         em(1) = (1.0d0-ci)/2.0d0
         em(2) = (1.0d0+ci)/2.0d0
         em(3) = dcmplx(0.0d0, 0.0d0)
      else
         fac = 1.0d0/(dsq2*kh*k)
         em(1) = (kx*kz-ci*ky*k*dsign(1.0d0, kz))*fac
         em(2) = (ky*kz+ci*kx*k*dsign(1.0d0, kz))*fac
         em(3) = -kh/(dsq2*k)
      end if

      return
      end
!---------------------------------------------------------------------
!- get the total nonlinear term for velocity profile
!---------------------------------------------------------------------
      subroutine totalnonlinear(total, u, irk)
      include 'head.h'

      complex*16 total(nx/2, ny, nz, 3)
      complex*16 u(nx/2, ny, nz, 3)
      integer irk

      complex*16 nl(nx/2, ny, nz, 3)
      common /vectortemp4/ nl
      complex*16 f(nx/2, ny, nz, 3)
      common /vectortemp5/ f
      complex*16 up(nx/2, ny, nz, 3)
      common /vectortemp6/ up

      integer ix, iy, iz
      real*8 rk2

!---- arrays of wavenumber ----
      real*8 k(nx/2, ny, nz, 3)
      common /wavenumber/ k

!- backup velocity
      call arraycopy_3d(up, u)

!- 1, get nonlinear term
      call nonlinear(nl, up)

!- 2, get forcing term
      if (iforce .ne. 0 .and. irk .eq. 1) then
         call forcing(f, u)
      else
         do iz = 1, nz
         do iy = 1, ny
         do ix = 1, nx/2
            f(ix,iy,iz,1) = dcmplx(0.0d0, 0.0d0)
            f(ix,iy,iz,2) = dcmplx(0.0d0, 0.0d0)
            f(ix,iy,iz,3) = dcmplx(0.0d0, 0.0d0)
         end do
         end do
         end do
      end if

!- 3, setup total nonlinear term
      do iz = 1, nz
      do iy = 1, ny
      do ix = 1, nx/2

         total(ix, iy, iz, 1) = -nl(ix, iy, iz, 1)+f(ix, iy, iz, 1)
         total(ix, iy, iz, 2) = -nl(ix, iy, iz, 2)+f(ix, iy, iz, 2)
         total(ix, iy, iz, 3) = -nl(ix, iy, iz, 3)+f(ix, iy, iz, 3)

      end do
      end do
      end do

      return
      end
!---------------------------------------------------------------------
!- calculate the nonlinear term
!---------------------------------------------------------------------
      subroutine nonlinear(nl, u)
      include 'head.h'

      real*8 nl(nx, ny, nz, 3)
      real*8 u(nx, ny, nz, 3)

      integer ix, iy, iz

      real*8 nl1, nl2, nl3

!- get vorticity
      call curl(nl, u)

!- put them back to real space
      call cr_3d(nl)
      call cr_3d(u)

! get the nonlinear term in real space
      etot(3) = 0.0d0
      etot(4) = 0.0d0
      do iz = 1, nz
      do iy = 1, ny
      do ix = 1, nx

         etot(3) = etot(3)+
     &(nl(ix,iy,iz,1)**2+nl(ix,iy,iz,2)**2+nl(ix,iy,iz,3)**2)
         etot(4) = etot(4)+nl(ix,iy,iz,3)**2
         nl1=nl(ix,iy,iz,2)*u(ix,iy,iz,3)-nl(ix,iy,iz,3)*u(ix,iy,iz,2)
         nl2=nl(ix,iy,iz,3)*u(ix,iy,iz,1)-nl(ix,iy,iz,1)*u(ix,iy,iz,3)
         nl3=nl(ix,iy,iz,1)*u(ix,iy,iz,2)-nl(ix,iy,iz,2)*u(ix,iy,iz,1)

         nl(ix, iy, iz, 1) = nl1
         nl(ix, iy, iz, 2) = nl2
         nl(ix, iy, iz, 3) = nl3

      end do
      end do
      end do
      etot(3) = etot(3)/dble(nx*ny*nz)
      etot(3) = dsqrt(etot(3))
      etot(4) = etot(4)/dble(nx*ny*nz)
      etot(4) = dsqrt(etot(4))

! put the nonlinear term into fourier space
      call rc_3d(nl)

      return
      end
!---------------------------------------------------------------------
!- add forcing
!---------------------------------------------------------------------
      subroutine forcing(f, u)
      include 'head.h'

      complex*16 f(nx/2, ny, nz, 3)
      complex*16 u(nx/2, ny, nz, 3)

      integer ipope
      common /pope/ ipope

      if (iforce .eq. 1) then

!         call pope_forcing1(f)

      else if (iforce .eq. 2) then

         call pope_forcing2(f, u)

      else if (iforce .eq. 12) then

         call pope_forcing2_m1(f, u)

      else if (iforce .eq. 5) then
!- the better one as a personal experience
         call rforce_vel_m1(f)

      end if

      return
      end
!---------------------------------------------------------------------
!- the first pope's forcing scheme, Eswaran-Pope, 1988
!---------------------------------------------------------------------
!      subroutine pope_forcing1(f)
!      include 'head.h'

!      complex*16 f(nx/2, ny, nz, 3)

!!---- arrays of wavenumber ----
!      real*8 k(nx/2, ny, nz, 3)
!      common /wavenumber/ k

!      integer ipope
!      common /pope/ ipope

!      integer ix, iy, iz, i

!      real*8 sd, rk2, rk, t_l, rand, acce

!      real*8 uo(nx/2, ny, nz, 6)
!      common /uoprocess/ uo

!      t_l = pi/(4.0d0*romega*ros(3))
!      sd = dsqrt(epsf*0.01d0/t_l)

!      if (ipope .eq. 0) then
!         uo = sd
!         ipope = 1
!      end if

!!- forcing begin
!      do iz = 1, nz
!      do iy = 1, ny
!      do ix = 1, nx/2

!         rk2 = k(ix,iy,iz,1)**2+k(ix,iy,iz,2)**2+k(ix,iy,iz,3)**2
!         rk = dsqrt(rk2)

!         if (rk > flow .and. rk < fhigh) then

!            do i = 1, 6
!            call own_random_number(rand)
!            acce = (rand-0.5d0)*2.0d0*sd
!            uo(ix,iy,iz,i) = dexp(-dt/t_l)*uo(ix,iy,iz,i)+acce*0.1d0
!            end do

!            f(ix,iy,iz,1) = dcmplx(uo(ix,iy,iz,1), uo(ix,iy,iz,2))
!            f(ix,iy,iz,2) = dcmplx(uo(ix,iy,iz,3), uo(ix,iy,iz,4))
!            f(ix,iy,iz,3) = dcmplx(uo(ix,iy,iz,5), uo(ix,iy,iz,6))

!         end if

!      end do
!      end do
!      end do

!      call project(f, dt)

!      return
!      end
!---------------------------------------------------------------------
!- the second pope's forcing scheme, Overholt-Pope, 1998
!---------------------------------------------------------------------
      subroutine pope_forcing2(f, u)
      include 'head.h'

      complex*16 f(nx/2, ny, nz, 3)
      complex*16 u(nx/2, ny, nz, 3)

!---- arrays of wavenumber ----
      real*8 k(nx/2, ny, nz, 3)
      common /wavenumber/ k

      real*8 spec(smax, 3)
      common /spectrumfield/ spec

      integer ipope
      common /pope/ ipope

      integer kpoint(nx/2, ny, nz)
      common /idwavenumber/ kpoint

      integer ix, iy, iz, i, kk
      integer inx, iny, inz
      common /forcingrange/ inx, iny, inz

      real*8 rk2, rk, Em, temp
      real*8 alpha, alpha_c, zf, psk, spec_bar, T_f

      real*8 tau_f
      common /ftime/ tau_f
      real*8 scalar_f(nx/2, ny, nz)
      common /fscalar/ scalar_f

!- array for the hyper and hypo viscosity
      real*8 hyper(nx/2, ny, nz), specmax
      common /hyperblock/ hyper, specmax

      real*8 kmax, rnutmp
      common /subtmp/ kmax, rnutmp

      if (ipope .eq. 0) then

      do iz = 1, nz
      do iy = 1, ny
      do ix = 1, nx/2
         scalar_f(ix, iy, iz) = 0.0d0
      end do
      end do
      end do

      tau_f = dsqrt(rnu/dtot)
      if (ihyper .eq. 1) then
         kmax = dble(min(nx,min(ny,nz)))/3.0d0
         rnutmp = 2.5d0*dsqrt(1.0d0/kmax)*kmax**(2-2*ihpower)
         tau_f =
     &((rnutmp*specmax)/dtot**ihpower)**(1.0d0/(3.0d0*ihpower-1.0d0))
      end if

      ipope = 1

      i = int(fhigh)+1
      inx = min(nx/2, i)
      iny = min(ny, i)
      inz = min(nz, i)

      end if

! forcing begin
      T_f = 0.4d0
      tau_f = dsqrt(rnu/dtot)*T_f
      if (ihyper .eq. 1) then
         tau_f =
     &((rnutmp*specmax)/dtot**ihpower)**(1.0d0/(3.0d0*ihpower-1.0d0))
      end if
      tau_f = max(tau_f, 1.0d0)
      tau_f = min(tau_f, 9.9d0)

      alpha = 0.7d0
      alpha_c = 2.0d0*dsqrt(2.0d0)/tau_f
      do iz = 1, inz
      do iy = 1, iny
      do ix = 1, inx

         rk2 = k(ix,iy,iz,1)**2+k(ix,iy,iz,2)**2+k(ix,iy,iz,3)**2
         rk = dsqrt(rk2)
         if (rk > flow .and. rk < fhigh) then

            kk = kpoint(ix, iy, iz)

            call cutoff(zf, rk, fhigh)
            spec_bar = spec(kk, 2)
            call model_spectrum(Em, rk)
            Em = Em*epsf
            temp = -log(spec_bar/(Em*zf))/tau_f-
     &alpha*alpha_c*scalar_f(ix, iy, iz)

            call operator(psk, temp, scalar_f(ix, iy, iz))

            scalar_f(ix, iy, iz) = scalar_f(ix, iy, iz)+psk*dt
            if (scalar_f(ix, iy, iz) < 0.0d0) then
               scalar_f(ix, iy, iz) = 0.0d0
            end if

            do i = 1, 3
               f(ix,iy,iz,i) = scalar_f(ix,iy,iz)*u(ix,iy,iz,i)/tau_f
            end do

         else
         	
            do i = 1, 3
               f(ix,iy,iz,i) = dcmplx(0.0d0, 0.0d0)
            end do
         
         end if

      end do
      end do
      end do

      call project(f, dt)

      return
      end
!---------------------------------------------------------------------
!- cutoff function for pope forcing 2
!---------------------------------------------------------------------
      subroutine cutoff(zf, k, kf)

      real*8 zf, k, kf

      real*8 zeta

      zeta = 0.25d0

      if (k > kf) then
            zf = 0.0d0
      else
            zf = dtanh((kf-k)/(zeta*kf))
      end if

      return
      end
!---------------------------------------------------------------------
!- operator P for pope forcing 2
!---------------------------------------------------------------------
      subroutine operator(psk, sk, f)
      real*8 psk, sk, f

      psk = 0.0d0
      if (f > 0.0d0 .or. sk > 0.0d0) then
            psk = sk
      end if

      return
      end
!---------------------------------------------------------------------
      subroutine model_spectrum(Em, k)

      real*8 Em, k

      real*8 fl, feta

      real*8 cl, ceta, C, dtot, L, eta, beta, p0
      parameter (cl = 0.97d0, ceta = 2.0d0, C = 1.5d0, dtot = 0.1d0)
      parameter (L = 1.0d0/0.4d0, eta = 1.0d0/40.0d0)
      parameter (beta = 5.2d0, p0 = 4.0d0)

      fl = (k*L/dsqrt((k*L)*(k*L)+cl) )**(5.0d0/3.0d0+p0)
      feta = dexp(-beta*(((k*eta)**4+ceta**4)**(1.0d0/4.0d0)-ceta))

      Em = C*(dtot**(2.0d0/3.0d0))*(k**(-5.0d0/3.0d0))*fl*feta
      return
      end
!---------------------------------------------------------------------
! the Smith and Waleffe's forcing scheme, modified by Lu, version 1
!---------------------------------------------------------------------
      subroutine rforce_vel_m1(f)
      include 'head.h'

      complex*16 f(nx/2, ny, nz, 3)

!---- arrays of wavenumber ----
      real*8 k(nx/2, ny, nz, 3)
      common /wavenumber/ k

      integer ix, iy, iz, i, kk
      integer inx, iny, inz
      common /forcingrange/ inx, iny, inz

      real*8 spec(nmax), irforce
      common /spec_rforce/ spec, irforce

      real*8 sd, rk2, rk, rkh, rkk, anorm, tmp
      complex*16 em(3), ep(3), bm, bp

      if (irforce .ne. 1.0d0) then
         sd = 1.0d0
         do i = 1, nmax
            rk = dble(i-1)+0.5d0
            rkk = (rk-fpeak)*(rk-fpeak)/(sd*sd)
            spec(i) = epsf*dexp(-0.5d0*rkk)/(dsqrt(2.0d0*pi)*sd)
            spec(i) = dsqrt(spec(i)/(2.0d0*pi*rk*rk))
         end do
         irforce = 1.0d0
         i = int(fhigh)+1
         inx = min(nx/2, i)
         iny = min(ny, i)
         inz = min(nz, i)
      end if

      tmp = dsqrt(dt)
!- forcing begin
      do iz = 1, inz
      do iy = 1, iny
      do ix = 1, inx

         rkh = k(ix, iy, iz, 1)**2+k(ix, iy, iz, 2)**2
         rk2 = rkh+k(ix, iy, iz, 3)**2
         rk = dsqrt(rk2)
         kk = int(rk)
         rkh = dsqrt(rkh)

         if (rk >=  flow .and. rk <=  fhigh) then

            bm = dcmplx(anorm(), anorm())*spec(kk)/tmp
            bp = dcmplx(anorm(), anorm())*spec(kk)/tmp

            call eigm(k(ix,iy,iz,1),k(ix,iy,iz,2),
     &k(ix,iy,iz,3),rkh,rk,em)

            do i = 1, 3
               ep(i) = dconjg(em(i))
            end do

!- f is not accumulating in this modified scheme
            do i = 1, 3
                f(ix, iy, iz, i) = bm*em(i)+ bp*ep(i)
            end do

            bm=ep(1)*f(ix,iy,iz,1)+ep(2)*f(ix,iy,iz,2)+
     &ep(3)*f(ix,iy,iz,3)
            bp=em(1)*f(ix,iy,iz,1)+em(2)*f(ix,iy,iz,2)+
     &em(3)*f(ix,iy,iz,3)

            do i = 1, 3
               f(ix, iy, iz, i) = bm*em(i)+bp*ep(i)
            end do
            
         else

            do i = 1, 3
               f(ix, iy, iz, i) = dcmplx(0.0d0, 0.0d0)
            end do

         end if

      end do
      end do
      end do

      return
      end
!---------------------------------------------------------------------
!- the second pope's forcing scheme, Overholt-Pope, 1998,
!- revised by Hao Lu 11/30/2005
!---------------------------------------------------------------------
      subroutine pope_forcing2_m1(f, u)
      include 'head.h'

      complex*16 f(nx/2, ny, nz, 3)
      complex*16 u(nx/2, ny, nz, 3)

!---- arrays of wavenumber ----
      real*8 k(nx/2, ny, nz, 3)
      common /wavenumber/ k

      real*8 spec(smax, 3)
      common /spectrumfield/ spec

      integer ipope
      common /pope/ ipope

      integer kpoint(nx/2, ny, nz)
      common /idwavenumber/ kpoint

      integer ix, iy, iz, i, kk
      integer inx, iny, inz
      common /forcingrange/ inx, iny, inz

      real*8 rk2, rk, Em, temp
      real*8 alpha, alpha_c, zf, psk, spec_bar, T_f

      real*8 tau_f
      common /ftime/ tau_f
      real*8 scalar_f(nx/2, ny, nz)
      common /fscalar/ scalar_f

!- array for the hyper and hypo viscosity
      real*8 hyper(nx/2, ny, nz), specmax
      common /hyperblock/ hyper, specmax

      real*8 kmax, rnutmp
      common /subtmp/ kmax, rnutmp

      if (ipope .eq. 0) then

      do iz = 1, nz
      do iy = 1, ny
      do ix = 1, nx/2
         scalar_f(ix, iy, iz) = 0.0d0
      end do
      end do
      end do

      tau_f = dsqrt(rnu/dtot)
      if (ihyper .eq. 1) then
         kmax = dble(min(nx,min(ny,nz)))/3.0d0
         rnutmp = 2.5d0*dsqrt(1.0d0/kmax)*kmax**(2-2*ihpower)
         tau_f =
     &((rnutmp*specmax)/dtot**ihpower)**(1.0d0/(3.0d0*ihpower-1.0d0))
      end if

      ipope = 1

      i = int(fhigh)+1
      inx = min(nx/2, i)
      iny = min(ny, i)
      inz = min(nz, i)

      end if

! forcing begin
      T_f = 0.4d0
      tau_f = dsqrt(rnu/dtot)*T_f
      if (ihyper .eq. 1) then
         tau_f =
     &((rnutmp*specmax)/dtot**ihpower)**(1.0d0/(3.0d0*ihpower-1.0d0))
      end if
      tau_f = max(tau_f, 1.0d0)
      tau_f = min(tau_f, 9.9d0)

      alpha = 0.7d0
      alpha_c = 2.0d0*dsqrt(2.0d0)/tau_f
      do iz = 1, inz
      do iy = 1, iny
      do ix = 1, inx

         rk2 = k(ix,iy,iz,1)**2+k(ix,iy,iz,2)**2+k(ix,iy,iz,3)**2
         rk = dsqrt(rk2)
         if (rk > flow .and. rk < fhigh) then

            kk = kpoint(ix, iy, iz)

            call cutoff(zf, rk, fhigh)
            spec_bar = spec(kk, 2)
            call model_spectrum(Em, rk)
            Em = Em*epsf
            temp = -log(spec_bar/(Em*zf))/tau_f-
     &alpha*alpha_c*scalar_f(ix, iy, iz)

            call operator(psk, temp, scalar_f(ix, iy, iz))

            scalar_f(ix, iy, iz) = scalar_f(ix, iy, iz)+psk*dt
            if (scalar_f(ix, iy, iz) < 0.0d0) then
               scalar_f(ix, iy, iz) = 0.0d0
            end if

            if (spec_bar > Em) then

               do i = 1, 3
               f(ix,iy,iz,i) = dcmplx(0.0d0,0.0d0)
               end do

            else

               do i = 1, 3
               f(ix,iy,iz,i) = scalar_f(ix,iy,iz)*u(ix,iy,iz,i)/tau_f
               end do

            end if

         else

            do i = 1, 3
               f(ix,iy,iz,i) = dcmplx(0.0d0, 0.0d0)
            end do
         
         end if

      end do
      end do
      end do

      call project(f, dt)

      return
      end
!---------------------------------------------------------------------
