!---------------------------------------------------------------------
!- Standard DNS code version 3.0 by Hao Lu 10/10/2005
!- solving the 3D incompressible Navier-Stokes equation.
!-
!- History and features:
!- 1, version 1 is provided on 6/30/2004. the ideal is from the revice
!- of Smith's code.
!-
!- 2, version 2.1 is provided on 12/29/2004. which is including pope's
!- forcing scheme.
!-
!- 3, version 2.2 includes two pope's forcing scheme, second one is
!- tested for lots of cases, not yet for first one. Smith's white
!- noise forcing scheme is provided here, and has been tested for lots
!- of case. Bugs on calculation of spectrums are fixed. Scalar
!- transportation is provided in the code. Rotation is allowed in the
!- code. Hypo viscosity is provided in the code. 2/2/2005
!-
!- 4, version 2.3 fixed the bugs on three Gaussian forcing schemes.
!- Optimized the step output. 3/11/2005
!-
!- 5, version 2.4 using integral factor/linear eigenmode to solve
!- the rotational term exactly. Code is more stable. 8/26/2005
!-
!- 6, version 3.0 has own fft code instead of using SGI math library's
!- fft. So the code can run at PC also. 10/10/2005
!---------------------------------------------------------------------
      program ns3d
      include 'head.h'

      complex*16 u(nx/2, ny, nz, 3)
      common /ublock/ u

!---- arrays of wavenumber ----
      real*8 k(nx/2, ny, nz, 3)
      common /wavenumber/ k

!- array for the hyper and hypo viscosity
      real*8 hyper(nx/2, ny, nz), specmax
      common /hyperblock/ hyper, specmax
      real*8 hypo(nx/2, ny, nz), specmin
      common /hypoblock/ hypo, specmin

      real*8 time1, time2

      character*99 filename, tail

      integer istep
      common /steps/ istep

      integer ioutput

      integer modstep

      integer schecktmp, scheckdelay

!- initialize or load the velocity and scalar profile
      write(*,*) 'begining initialization...'

      scheckdelay = 100
      pi = 4.0d0*datan(1.0d0)

      call zbqlini(2005)
      call fftini()
      call readparams()
      call kvector(k)
      call setupvisc(hyper, hypo)

      call init(u)

      if (istart .eq. 0 .or. istart .eq. 1) then

      filename = 'evolution.dat'
      open(99,file=filename,status='replace',form='formatted')
      write(99,*) 
     &'#time step k kk taut re eta rosL roso roso3 rosla k2r'
      close(99)

      filename = 'evolution_spec.dat'
      open(99,file=filename,status='replace',form='formatted')
      write(99, '(1x,a\)')
     &'#time k1 k2 k3 k4 k5 k6 k7 k8 k9 k10 k11 k12 k13 k14 k15 k16'
      write(99, *) 'k17 k18 k19 k20 k21 k22 k23'
      close(99)

      end if

!---------------------------------------------------------------------
!- begin time-step loop
!---------------------------------------------------------------------
      write(*, *) 'begining to calculate...'
! output for plot3d
!      filename = 'geo3d.xyz'
!      call xyz_3d_output(filename)
!      filename = 'geo2d.xyz'
!      call xyz_2d_output(filename)

      call cpu_time(time1)

      schecktmp = scheck
      do istep = begin, tint

      if (istep < begin+scheckdelay) then
        scheck = 1
      else
        scheck = schecktmp
      end if

      call reform(u)
      call truncate_3d(u)

!- setup favorable time step to make stable
      if (modstep(istep,scheck) .eq. 1) then
        call checkstable(u)
      end if

!- output the spectrum
      if (modstep(istep,tmeas) .eq. 1 .and.
     &(istart .ne. 2 .or. istep > (tmeas+begin-1))) then
         call zerofill(tail,istep,6)
         filename = 'spectrum_u'
         call cdotappend(filename, tail)
         ioutput = 1
      else
         ioutput = 0
      end if

!- using hyper, hypo viscosity and pope's forcing, needs to calculate
!- the spectrum more frequently
      if (modstep(istep,scheck) .eq. 1 .and.
     &(ihyper .eq. 1 .or. ihypo .eq. 1 .or.
     &iforce .eq. 2 .or. iforce .eq. 12 .or. iforce .eq. 3)) then

         call spectrum_u(u, filename, ioutput)

      else if (modstep(istep,tmeas) .eq. 1 .and.
     &(istart .ne. 2 .or. istep > (tmeas+begin-1))) then

         call spectrum_u(u, filename, ioutput)

      end if

!- output the velocity profile
      if (modstep(istep,ncheck) .eq. 1 .and.
     &(istart .ne. 2 .or. istep > (ncheck+begin-1))) then
!      if (modstep(istep, ncheck) .eq. 1) then

      if (mode .eq. 0) then

! output for plot3d
!         call zerofill(tail,istep,6)
!         filename = 'velocity3d'
!         call cdotappend(filename, tail)
!         tail = 'q'
!         call cdotappend(filename, tail)
!         call q_3d_output(u, filename)
!         call zerofill(tail,istep,6)
!         filename = 'velocity2d'
!         call cdotappend(filename, tail)
!         tail = 'q'
!         call cdotappend(filename, tail)
!         call q_2d_output(u, filename)

         call zerofill(tail,istep,6)
         filename = 'velocity'
         call cdotappend(filename, tail)
         call velocity_b_output(u, filename)

      else if (mode .eq. 2 .or. mode .eq. 3) then

         call zerofill(tail,istep,6)
         filename = 'velocity4'
         call cdotappend(filename, tail)
         call velocity_b4_output(u, filename)

      end if

      end if

!---------------------------------------------------------------------
!- update the velocity by 3rd order runge-kutta scheme
!---------------------------------------------------------------------
      call update3(u)
!---------------------------------------------------------------------

      call cpu_time(time2)

      write(*, 10)time, istep, dt, time2-time1, etot(1)
10    format('t=', f11.4, '|stp=', i6, '|dt=', f9.6, '|cpu cost:',
     &f7.3, '|K=', f11.5)

      time1 = time2

      time = time+dt

      end do

      end
!---------------------------------------------------------------------
